==> 0.3.0.tar.gz: lib/src/modules/gestures/gesture_listener.dart <==
/// This file is inspired from gesture_x_detector.dart from the [gesture_x_detector](https://github.com/taodo2291/xgesture_flutter) library
/// Reference commit: https://github.com/taodo2291/xgesture_flutter/tree/f57721bed8194021f36b3e454ef7b828bf259c53

import 'dart:async';

import 'package:flutter/gestures.dart';
import 'package:flutter/widgets.dart';

/// Event describing a "Move" type gesture
class MoveEvent {
  /// Location on screen where the [MoveEvent] started from
  late Offset startLocation;

  /// Delta by which the pointer has moved
  late Offset delta;

  /// Speed at which the pointer has moved
  late Offset velocity;

  /// Creates a move event
  MoveEvent({
    required this.startLocation,
    required this.delta,
    required this.velocity,
  });

  /// Creates a move event from a given [_Touch]
  MoveEvent._fromTouch(_Touch touch) {
    startLocation = touch.startPosition;
    delta = touch.currentPosition - touch.startPosition;
    velocity = touch.velocityTracker.getVelocityEstimate()?.pixelsPerSecond ??
        Offset.zero;
  }
}

/// Signature for listening to tap events.
typedef TapEventListener = void Function(PointerEvent event);

/// Signature for listening to move events.
typedef MoveEventListener = void Function(MoveEvent event);

///  A widget that detects gestures.
/// * Supports Tap, Long Press & Move(start, update, end)
/// * All callbacks be used simultaneously
/// * Event is not absorbed
class GestureListener extends StatefulWidget {
  /// Creates a widget that detects gestures.
  const GestureListener({
    Key? key,
    required this.child,
    this.onTap,
    this.onLongPress,
    this.onLongPressEnd,
    this.onMoveStart,
    this.onMoveUpdate,
    this.onMoveEnd,
    this.longPressTimeout = kLongPressTimeout,
    this.touchSlop = kTouchSlop,
    this.bypassMoveEventAfterLongPress = true,
  }) : super(key: key);

  /// The widget below this widget in the tree.
  ///
  /// {@macro flutter.widgets.child}
  final Widget child;

  /// Callback for a detected "Tap" gesture:
  /// A single [PointerEvent.pointer] has sequentially triggered a [PointerDownEvent] followed by a [PointerUpEvent] without moving further than [kTouchSlop]
  final TapEventListener? onTap;

  /// Callback for a detected "Long Press" gesture:
  /// A single [PointerEvent.pointer] has sequentially triggered a [PointerDownEvent] followed by a [PointerUpEvent] without moving further than [kTouchSlop]
  /// while remaining in contact with the screen for a duration > [longPressTimeout]
  final TapEventListener? onLongPress;

  /// Callback triggered after a "Long press" gesture is released:
  final Function()? onLongPressEnd;

  /// Callback for when a pointer, in contact with the screen, has started to move
  /// A single [PointerEvent.pointer] has sequentially triggered a [PointerDownEvent] followed by a series of [PointerMoveEvent], moving further than [kTouchSlop]
  final MoveEventListener? onMoveStart;

  /// Callback for when a pointer, in contact with the screen has made a mmove
  /// A single [PointerEvent.pointer] has sequentially triggered a [PointerDownEvent] followed by a series of [PointerMoveEvent], moving further than [kTouchSlop]
  final MoveEventListener? onMoveUpdate;

  /// Callback for when a pointer that was previously in contact with the screen and moving is no longer in contact with the screen
  /// A single [PointerEvent.pointer] has sequentially triggered a [PointerDownEvent] followed by a series of [PointerMoveEvent], moving further than [kTouchSlop]
  /// eventually reaching a [PointerUpEvent] when the contact with the screen is broken
  final MoveEventListener? onMoveEnd;

  /// The minimal distance a pointer needs to move to actually be considered moving
  /// A pointer moving less that [touchSlop] is considered to be either a "Tap" or a "Long Press"
  /// depending on the duration
  final double touchSlop;

  /// A specific duration to detect long press
  final Duration longPressTimeout;

  ///  by default it is true, means after receive long press event without release pointer (finger still touch on screen)
  /// the move event will be ignore.
  ///
  /// set it to false in case you expect move event will be fire after long press event
  ///
  final bool bypassMoveEventAfterLongPress;

  @override
  _GestureListenerState createState() => _GestureListenerState();
}

enum _GestureState {
  pointerDown,
  moveStart,
  longPress,
  unknown,
}

class _Touch {
  late Offset startPosition;
  late Offset currentPosition;
  late VelocityTracker velocityTracker;

  Offset get delta => currentPosition - startPosition;

  _Touch.fromPointerEvent(PointerEvent event) {
    startPosition = event.localPosition;
    currentPosition = startPosition;
    velocityTracker = VelocityTracker.withKind(event.kind)
      ..addPosition(event.timeStamp, event.position);
  }
}

class _GestureListenerState extends State<GestureListener> {
  final Map<int, _Touch> _touches = <int, _Touch>{};
  _GestureState state = _GestureState.unknown;
  Timer? longPressTimer;
  Offset lastTouchUpPos = Offset.zero;

  @override
  Widget build(BuildContext context) {
    return Listener(
      behavior: HitTestBehavior.translucent,
      child: widget.child,
      onPointerDown: onPointerDown,
      onPointerUp: onPointerUp,
      onPointerMove: onPointerMove,
      onPointerCancel: onPointerCancel,
    );
  }

  @override
  void dispose() {
    _stopTimer();
    _touches.clear();
    super.dispose();
  }

  void _giveUpPointer(int pointer) {
    _touches.remove(pointer);
  }

  void onPointerDown(PointerDownEvent event) {
    _touches[event.pointer] = _Touch.fromPointerEvent(event);

    if (touchCount == 1) {
      state = _GestureState.pointerDown;
      startLongPressTimer(event);
    } else {
      state = _GestureState.unknown;
    }
  }

  void onPointerMove(PointerMoveEvent event) {
    final touch = _touches[event.pointer];

    if (touch == null) return;

    touch.currentPosition = event.localPosition;

    if (!isWithinTouchSlop(touch)) {
      _stopTimer();
    }

    switch (state) {
      case _GestureState.longPress:
        if (widget.bypassMoveEventAfterLongPress) {
          touch.startPosition = touch.currentPosition;
        } else {
          switch2MoveStartState(touch, event);
        }
        break;
      case _GestureState.pointerDown:
        switch2MoveStartState(touch, event);
        break;
      case _GestureState.moveStart:
        touch.velocityTracker.addPosition(event.timeStamp, event.position);
        widget.onMoveUpdate?.call(MoveEvent._fromTouch(touch));
        break;

      default:
        touch.startPosition = touch.currentPosition;
        break;
    }
  }

  void switch2MoveStartState(_Touch touch, PointerMoveEvent event) {
    touch.currentPosition = event.localPosition;
    if (!isWithinTouchSlop(touch)) {
      state = _GestureState.moveStart;
      widget.onMoveStart?.call(MoveEvent._fromTouch(touch));
    }
  }

  void onPointerUp(PointerUpEvent event) {
    final touch = _touches[event.pointer];
    if (touch == null) return;

    touch.currentPosition = event.position;
    if (state == _GestureState.pointerDown) {
      callOnTap(event.copyWith(position: touch.startPosition));
    } else if (state == _GestureState.moveStart) {
      state = _GestureState.unknown;
      touch.velocityTracker.addPosition(event.timeStamp, event.position);

      widget.onMoveEnd?.call(MoveEvent._fromTouch(touch));
    } else if (state == _GestureState.longPress) {
      widget.onLongPressEnd?.call();
      state = _GestureState.unknown;
    } else {
      state = _GestureState.unknown;
    }

    lastTouchUpPos = event.localPosition;
    _giveUpPointer(event.pointer);
  }

  void onPointerCancel(PointerCancelEvent event) =>
      _giveUpPointer(event.pointer);

  void startLongPressTimer(PointerEvent event) {
    if (widget.onLongPress != null) {
      longPressTimer?.cancel();
      longPressTimer = Timer(widget.longPressTimeout, () {
        if (touchCount == 1 && _touches.entries.first.key == event.pointer) {
          state = _GestureState.longPress;
          widget.onLongPress!(event);
          _stopTimer();
        }
      });
    }
  }

  bool isWithinTouchSlop(_Touch touch) {
    return touch.delta.distance.abs() <= widget.touchSlop;
  }

  void _stopTimer() {
    longPressTimer?.cancel();
    longPressTimer = null;
  }

  void callOnTap(PointerEvent event) {
    widget.onTap?.call(event);
  }

  get touchCount => _touches.length;
}

==> 0.5.1.tar.gz: lib/src/modules/gestures/gesture_listener.dart <==
/// This file is inspired from gesture_x_detector.dart from the [gesture_x_detector](https://github.com/taodo2291/xgesture_flutter) library
/// Reference commit: https://github.com/taodo2291/xgesture_flutter/tree/f57721bed8194021f36b3e454ef7b828bf259c53

import 'dart:async';

import 'package:flutter/gestures.dart';
import 'package:flutter/widgets.dart';

/// Event describing a "Move" type gesture
class MoveEvent {
  /// Location on screen where the [MoveEvent] started from
  final Offset startLocation;

  /// Delta by which the pointer has moved
  final Offset delta;

  /// Speed at which the pointer has moved
  final Offset velocity;

  /// Creates a move event from a given [_Touch]
  MoveEvent._fromTouch(_Touch touch)
      : startLocation = touch.startPosition,
        delta = touch.currentPosition - touch.startPosition,
        velocity =
            touch.velocityTracker.getVelocityEstimate()?.pixelsPerSecond ??
                Offset.zero;
}

/// Signature for listening to tap events.
typedef TapEventListener = void Function(PointerEvent event);

/// Signature for listening to move events.
typedef MoveEventListener = void Function(MoveEvent event);

///  A widget that detects gestures.
/// * Supports Tap, Long Press & Move(start, update, end)
/// * All callbacks be used simultaneously
/// * Event is not absorbed
class GestureListener extends StatefulWidget {
  /// Creates a widget that detects gestures.
  const GestureListener({
    Key? key,
    required this.child,
    this.onTap,
    this.onLongPress,
    this.onLongPressEnd,
    this.onMoveStart,
    this.onMoveUpdate,
    this.onMoveEnd,
    this.longPressTimeout = kLongPressTimeout,
    this.touchSlop = kTouchSlop,
    this.bypassMoveEventAfterLongPress = true,
  }) : super(key: key);

  /// The widget below this widget in the tree.
  ///
  /// {@macro flutter.widgets.child}
  final Widget child;

  /// Callback for a detected "Tap" gesture:
  /// A single [PointerEvent.pointer] has sequentially triggered a [PointerDownEvent] followed by a [PointerUpEvent] without moving further than [kTouchSlop]
  final TapEventListener? onTap;

  /// Callback for a detected "Long Press" gesture:
  /// A single [PointerEvent.pointer] has sequentially triggered a [PointerDownEvent] followed by a [PointerUpEvent] without moving further than [kTouchSlop]
  /// while remaining in contact with the screen for a duration > [longPressTimeout]
  final TapEventListener? onLongPress;

  /// Callback triggered after a "Long press" gesture is released:
  final Function()? onLongPressEnd;

  /// Callback for when a pointer, in contact with the screen, has started to move
  /// A single [PointerEvent.pointer] has sequentially triggered a [PointerDownEvent] followed by a series of [PointerMoveEvent], moving further than [kTouchSlop]
  final MoveEventListener? onMoveStart;

  /// Callback for when a pointer, in contact with the screen has made a mmove
  /// A single [PointerEvent.pointer] has sequentially triggered a [PointerDownEvent] followed by a series of [PointerMoveEvent], moving further than [kTouchSlop]
  final MoveEventListener? onMoveUpdate;

  /// Callback for when a pointer that was previously in contact with the screen and moving is no longer in contact with the screen
  /// A single [PointerEvent.pointer] has sequentially triggered a [PointerDownEvent] followed by a series of [PointerMoveEvent], moving further than [kTouchSlop]
  /// eventually reaching a [PointerUpEvent] when the contact with the screen is broken
  final MoveEventListener? onMoveEnd;

  /// The minimal distance a pointer needs to move to actually be considered moving
  /// A pointer moving less that [touchSlop] is considered to be either a "Tap" or a "Long Press"
  /// depending on the duration
  final double touchSlop;

  /// A specific duration to detect long press
  final Duration longPressTimeout;

  ///  by default it is true, means after receive long press event without release pointer (finger still touch on screen)
  /// the move event will be ignore.
  ///
  /// set it to false in case you expect move event will be fire after long press event
  ///
  final bool bypassMoveEventAfterLongPress;

  @override
  _GestureListenerState createState() => _GestureListenerState();
}

enum _GestureState {
  pointerDown,
  moveStart,
  longPress,
  unknown,
}

class _Touch {
  late Offset startPosition;
  late Offset currentPosition;
  late VelocityTracker velocityTracker;

  Offset get delta => currentPosition - startPosition;

  _Touch.fromPointerEvent(PointerEvent event) {
    startPosition = event.localPosition;
    currentPosition = startPosition;
    velocityTracker = VelocityTracker.withKind(event.kind)
      ..addPosition(event.timeStamp, event.position);
  }
}

class _GestureListenerState extends State<GestureListener> {
  final Map<int, _Touch> _touches = <int, _Touch>{};
  _GestureState state = _GestureState.unknown;
  Timer? longPressTimer;
  Offset lastTouchUpPos = Offset.zero;

  @override
  Widget build(BuildContext context) {
    return Listener(
      behavior: HitTestBehavior.translucent,
      child: widget.child,
      onPointerDown: onPointerDown,
      onPointerUp: onPointerUp,
      onPointerMove: onPointerMove,
      onPointerCancel: onPointerCancel,
    );
  }

  @override
  void dispose() {
    _stopTimer();
    _touches.clear();
    super.dispose();
  }

  void _giveUpPointer(int pointer) {
    _touches.remove(pointer);
  }

  void onPointerDown(PointerDownEvent event) {
    _touches[event.pointer] = _Touch.fromPointerEvent(event);

    if (touchCount == 1) {
      state = _GestureState.pointerDown;
      startLongPressTimer(event);
    } else {
      state = _GestureState.unknown;
    }
  }

  void onPointerMove(PointerMoveEvent event) {
    final touch = _touches[event.pointer];

    if (touch == null) return;

    touch.currentPosition = event.localPosition;

    if (!isWithinTouchSlop(touch)) {
      _stopTimer();
    }

    switch (state) {
      case _GestureState.longPress:
        if (widget.bypassMoveEventAfterLongPress) {
          touch.startPosition = touch.currentPosition;
        } else {
          switch2MoveStartState(touch, event);
        }
        break;
      case _GestureState.pointerDown:
        switch2MoveStartState(touch, event);
        break;
      case _GestureState.moveStart:
        touch.velocityTracker.addPosition(event.timeStamp, event.position);
        widget.onMoveUpdate?.call(MoveEvent._fromTouch(touch));
        break;

      default:
        touch.startPosition = touch.currentPosition;
        break;
    }
  }

  void switch2MoveStartState(_Touch touch, PointerMoveEvent event) {
    touch.currentPosition = event.localPosition;
    if (!isWithinTouchSlop(touch)) {
      state = _GestureState.moveStart;
      widget.onMoveStart?.call(MoveEvent._fromTouch(touch));
    }
  }

  void onPointerUp(PointerUpEvent event) {
    final touch = _touches[event.pointer];
    if (touch == null) return;

    touch.currentPosition = event.position;
    if (state == _GestureState.pointerDown) {
      callOnTap(event.copyWith(position: touch.startPosition));
    } else if (state == _GestureState.moveStart) {
      state = _GestureState.unknown;
      touch.velocityTracker.addPosition(event.timeStamp, event.position);

      widget.onMoveEnd?.call(MoveEvent._fromTouch(touch));
    } else if (state == _GestureState.longPress) {
      widget.onLongPressEnd?.call();
      state = _GestureState.unknown;
    } else {
      state = _GestureState.unknown;
    }

    lastTouchUpPos = event.localPosition;
    _giveUpPointer(event.pointer);
  }

  void onPointerCancel(PointerCancelEvent event) =>
      _giveUpPointer(event.pointer);

  void startLongPressTimer(PointerEvent event) {
    if (widget.onLongPress != null) {
      longPressTimer?.cancel();
      longPressTimer = Timer(widget.longPressTimeout, () {
        if (touchCount == 1 && _touches.entries.first.key == event.pointer) {
          state = _GestureState.longPress;
          widget.onLongPress!(event);
          _stopTimer();
        }
      });
    }
  }

  bool isWithinTouchSlop(_Touch touch) {
    return touch.delta.distance.abs() <= widget.touchSlop;
  }

  void _stopTimer() {
    longPressTimer?.cancel();
    longPressTimer = null;
  }

  void callOnTap(PointerEvent event) {
    widget.onTap?.call(event);
  }

  get touchCount => _touches.length;
}
